==> scanbot_sdk-2.13.2-alpha02.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/camera/ScanbotCameraPlugin.kt <==
package io.scanbot.sdk.flutter.camera

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.barcode.ui.BarcodeScannerView
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.flutter.*
import io.scanbot.sdk.flutter.camera.DetectorType.*
import io.scanbot.sdk.flutter.camera.detectorWrappers.BarcodeDetector
import io.scanbot.sdk.flutter.camera.detectorWrappers.DocumentDetector
import io.scanbot.sdk.flutter.camera.detectorWrappers.IDetectorWrapper
import io.scanbot.sdk.flutter.camera.detectorWrappers.MedicalCertificateDetector
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageStorageProcessor
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch
import kotlin.coroutines.CoroutineContext


class ScanbotCameraPlatformView internal constructor(
    private val context: Context,
    private val messenger: BinaryMessenger,
    private val id: Int,
    private val creationParams: Json?
) : PlatformView,
    MethodChannel.MethodCallHandler, CoroutineScope {

    private var rootView: FrameLayout = FrameLayout(context)
    private val scanbotSDK = ScanbotSDK(context)
    private val customViewDisplayOwner: CustomViewDisplayOwner = CustomViewDisplayOwner()
    private var cameraChannel = MethodChannel(messenger, "scanbot_sdk_camera_$id")
    private val detectorChannel = MethodChannel(messenger, "scanbot_sdk_camera.detector_$id")
    private val finderChannel = MethodChannel(
        messenger,
        "scanbot_sdk_camera.finder_${creationParams?.get("layout_orientation") ?: ""}"
    )
    private var currentDetector: IDetectorWrapper? = null
    private val returnContext = Dispatchers.Main

    val gson = GsonBuilder()
        .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
        .registerTypeAdapter(ByteArray::class.java, ByteArrayToBase64TypeAdapter())
        .registerTypeAdapter(Page.Size::class.java, PageSizeAdapter())
        .registerTypeAdapter(PageStorageProcessor.Configuration.Size::class.java, PageProcessorSizeAdapter())
        .create()

    private var job: Job = Job()
    override val coroutineContext: CoroutineContext
        get() = Dispatchers.Default + job

    override fun getView(): View {
        return rootView
    }

    init {
        initCamera()
    }

    private fun initCamera() {
        cameraChannel.setMethodCallHandler(this)
        creationParams?.apply {
            checkParam<Json>("scannerConfiguration")?.let {
                attachDetector(it)
            }
        }
    }

    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {
        when (methodCall.method) {
            "flashEnabled" -> {
                val useFlash = methodCall.argument<Boolean>("enabled") ?: false
                currentDetector?.flashEnabled(useFlash)
                result.success("success")
            }

            "toggleContinuousFocus" -> {
                currentDetector?.toggleContinuousFocus()
                result.success("success")
            }

            "dispose" -> {
                customViewDisplayOwner.stopListening()
                dispose()
                result.success("success")
            }

            "stopPreview" -> {
                customViewDisplayOwner.stopListening()
                result.success("success")
            }

            "resumePreview" -> {
                customViewDisplayOwner.startListening()
                result.success("success")
            }

            "isFlashAvailable" -> {
                result.success(currentDetector?.isFlashAvailable() ?: false)
            }

            else -> result.notImplemented()
        }
    }

    private fun attachDetector(arguments: Json) {
        clearPrevDetectorState()
        if (checkLicenseInvalid()) return
        val detectorType = valueOf((arguments["detectorType"] as? String) ?: "")
        currentDetector = when (detectorType) {
            barcode -> BarcodeScannerView(context).let {
                rootView.addView(it)
                BarcodeDetector(
                    it, this,
                    cameraChannel,
                    detectorChannel,
                    finderChannel,
                    scanbotSDK,
                    customViewDisplayOwner,
                    creationParams ?: HashMap(),
                    gson
                )
            }

            mrz -> TODO("Mrz scanner is not implemented yet")
            ehic -> TODO("Health insurance scanner is not implemented yet")
            payform -> TODO("Payform scanner is not implemented yet")
            document -> DocumentScannerView(context).let {
                rootView.addView(it)
                DocumentDetector(
                    it, this,
                    cameraChannel,
                    detectorChannel,
                    scanbotSDK,
                    customViewDisplayOwner,
                    creationParams ?: HashMap(),
                    gson
                )
            }

            mc -> LayoutInflater.from(context).inflate(R.layout.scanbot_wrapper, rootView, false)
                .let {
                    rootView.addView(it)
                    MedicalCertificateDetector(
                        it as FrameLayout, this,
                        cameraChannel,
                        detectorChannel,
                        finderChannel,
                        scanbotSDK,
                        customViewDisplayOwner,
                        creationParams ?: HashMap(),
                        gson
                    )
                }
        }

        currentDetector?.attach(arguments)
        customViewDisplayOwner.startListening()

    }

    private fun checkLicenseInvalid(): Boolean {
        if (!scanbotSDK.licenseInfo.isValid) {
            launch(returnContext) {
                val json = scanbotSDK.getLicenseStatusJson()
                detectorChannel.invokeMethod(
                    "onLicenseError",
                    json.toString()
                )
            }
            return true
        }
        return false
    }

    private fun clearPrevDetectorState() {
        rootView.removeAllViews()
        currentDetector?.detach()
        currentDetector = null
    }


    override fun dispose() {
        clearPrevDetectorState()
        job.cancel()
        customViewDisplayOwner.stopListening()
    }

    override fun onFlutterViewDetached() {
        super.onFlutterViewDetached()
    }

    override fun onFlutterViewAttached(flutterView: View) {
        super.onFlutterViewAttached(flutterView)
    }

}

enum class DetectorType {
    barcode,
    mrz,
    mc,
    ehic,
    document,
    payform,
}

==> scanbot_sdk-4.1.0-alpha02.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/camera/ScanbotCameraPlugin.kt <==
package io.scanbot.sdk.flutter.camera

import android.app.Activity
import android.content.Context
import android.content.MutableContextWrapper
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.barcode.ui.BarcodeScannerView
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.flutter.*
import io.scanbot.sdk.flutter.camera.DetectorType.*
import io.scanbot.sdk.flutter.camera.detectorWrappers.BarcodeDetector
import io.scanbot.sdk.flutter.camera.detectorWrappers.DocumentDetector
import io.scanbot.sdk.flutter.camera.detectorWrappers.IDetectorWrapper
import io.scanbot.sdk.flutter.camera.detectorWrappers.MedicalCertificateDetector
import io.scanbot.sdk.flutter.utils.LifecycleOwnerWrapper
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageStorageProcessor
import io.scanbot.sdk.util.view.unwrapLifecycleOwner
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch
import kotlin.coroutines.CoroutineContext


class ScanbotCameraPlatformView internal constructor(
    private val context: Context,
    private val messenger: BinaryMessenger,
    private val id: Int,
    private val creationParams: Json?
) : PlatformView,
    MethodChannel.MethodCallHandler, CoroutineScope {

    private var rootView: FrameLayout = FrameLayout(context)
    private val scanbotSDK = ScanbotSDK(context)
    private val customViewDisplayOwner: CustomViewDisplayOwner = CustomViewDisplayOwner()
    private var cameraChannel = MethodChannel(messenger, "scanbot_sdk_camera_$id")
    private val detectorChannel = MethodChannel(messenger, "scanbot_sdk_camera.detector_$id")
    private val finderChannel = MethodChannel(
        messenger,
        "scanbot_sdk_camera.finder_${creationParams?.get("layout_orientation") ?: ""}"
    )
    private var currentDetector: IDetectorWrapper? = null
    private val returnContext = Dispatchers.Main

    val gson = GsonBuilder()
        .setFieldNamingPolicy(FieldNamingPolicy.IDENTITY)
        .registerTypeAdapter(ByteArray::class.java, ByteArrayToBase64TypeAdapter())
        .registerTypeAdapter(Page.Size::class.java, PageSizeAdapter())
        .registerTypeAdapter(
            PageStorageProcessor.Configuration.Size::class.java,
            PageProcessorSizeAdapter()
        )
        .create()

    private var job: Job = Job()
    override val coroutineContext: CoroutineContext
        get() = Dispatchers.Default + job

    override fun getView(): View {
        return rootView
    }

    init {
        initCamera()
    }

    private fun initCamera() {
        cameraChannel.setMethodCallHandler(this)
        creationParams?.apply {
            checkParam<Json>("scannerConfiguration")?.let {
                attachDetector(it)
            }
        }
    }

    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {
        when (methodCall.method) {
            "flashEnabled" -> {
                val useFlash = methodCall.argument<Boolean>("enabled") ?: false
                currentDetector?.flashEnabled(useFlash)
                result.success("success")
            }

            "toggleContinuousFocus" -> {
                currentDetector?.toggleContinuousFocus()
                result.success("success")
            }

            "dispose" -> {
                customViewDisplayOwner.stopListening()
                dispose()
                result.success("success")
            }

            "stopPreview" -> {
                customViewDisplayOwner.stopListening()
                result.success("success")
            }

            "resumePreview" -> {
                customViewDisplayOwner.startListening()
                result.success("success")
            }

            "isFlashAvailable" -> {
                result.success(currentDetector?.isFlashAvailable() ?: false)
            }

            else -> result.notImplemented()
        }
    }

    private fun attachDetector(arguments: Json) {
        clearPrevDetectorState()
        if (checkLicenseInvalid()) return
        val detectorType = valueOf((arguments["detectorType"] as? String) ?: "")
        currentDetector = when (detectorType) {
            barcode -> BarcodeScannerView(context).let {
                LifecycleOwnerWrapper(
                    context,
                    it,
                    (context as MutableContextWrapper).baseContext.unwrapLifecycleOwner()!!
                ).let { wrapper ->
                    rootView.addView(
                        wrapper,
                        FrameLayout.LayoutParams(
                            FrameLayout.LayoutParams.MATCH_PARENT,
                            FrameLayout.LayoutParams.MATCH_PARENT
                        )
                    )
                    BarcodeDetector(
                        wrapper,
                        this,
                        cameraChannel,
                        detectorChannel,
                        finderChannel,
                        scanbotSDK, customViewDisplayOwner,
                        creationParams ?: HashMap(),
                        gson
                    )
                }

            }

            mrz -> TODO("Mrz scanner is not implemented yet")
            ehic -> TODO("Health insurance scanner is not implemented yet")
            payform -> TODO("Payform scanner is not implemented yet")
            document -> DocumentScannerView(context).let {
                rootView.addView(it)
                DocumentDetector(
                    it, this,
                    cameraChannel,
                    detectorChannel,
                    scanbotSDK,
                    customViewDisplayOwner,
                    creationParams ?: HashMap(),
                    gson
                )
            }

            mc -> LayoutInflater.from(context).inflate(R.layout.scanbot_wrapper, rootView, false)
                .let {
                    rootView.addView(it)
                    MedicalCertificateDetector(
                        it as FrameLayout, this,
                        cameraChannel,
                        detectorChannel,
                        finderChannel,
                        scanbotSDK,
                        customViewDisplayOwner,
                        creationParams ?: HashMap(),
                        gson
                    )
                }
        }

        currentDetector?.attach(arguments)
        customViewDisplayOwner.startListening()

    }

    private fun checkLicenseInvalid(): Boolean {
        if (!scanbotSDK.licenseInfo.isValid) {
            launch(returnContext) {
                val json = scanbotSDK.getLicenseStatusJson()
                detectorChannel.invokeMethod(
                    "onLicenseError",
                    json.toString()
                )
            }
            return true
        }
        return false
    }

    private fun clearPrevDetectorState() {
        rootView.removeAllViews()
        currentDetector?.detach()
        currentDetector = null
    }


    override fun dispose() {
        clearPrevDetectorState()
        job.cancel()
        customViewDisplayOwner.stopListening()
    }

    override fun onFlutterViewDetached() {
        super.onFlutterViewDetached()
    }

    override fun onFlutterViewAttached(flutterView: View) {
        super.onFlutterViewAttached(flutterView)
    }

}

enum class DetectorType {
    barcode,
    mrz,
    mc,
    ehic,
    document,
    payform,
}
